@@ -1377,7 +1377,15 @@ A hook which binds F1 to `perldoc-at-point'." nil nil)
 
 ;;;***
 
-;;;### (autoloads (project-add) "projects" "projects.el" (18097 34336))
+;;;### (autoloads (pod-mode) "pod-mode" "pod-mode.el" (18252 12261))
+;;; Generated autoloads from pod-mode.el
+
+(autoload (quote pod-mode) "pod-mode" "\
+Major mode for editing POD files (Plain Old Documentation for Perl)." t nil)
+
+;;;***
+
+;;;### (autoloads (project-add) "projects" "projects.el" (18252 12219))
 ;;; Generated autoloads from projects.el
 
 (autoload (quote project-add) "projects" "\
@@ -1488,6 +1496,30 @@ Enable or disable tab-completion for some commands.
 The commands are `shell-command', `shell-command-on-region', `grep',
 `grep-find' and `compile'." t nil)
 
+;;;***
+
+;;;### (autoloads (show-ws-toggle-show-trailing-whitespace show-ws-toggle-show-hard-spaces
+;;;;;;  show-ws-toggle-show-tabs) "show-wspace" "show-wspace.el"
+;;;;;;  (18176 64695))
+;;; Generated autoloads from show-wspace.el
+
+(defalias (quote toggle-show-tabs-show-ws) (quote show-ws-toggle-show-tabs))
+
+(autoload (quote show-ws-toggle-show-tabs) "show-wspace" "\
+Toggle highlighting of TABs, using face `show-ws-tab'." t nil)
+
+(defalias (quote toggle-show-hard-spaces-show-ws) (quote show-ws-toggle-show-hard-spaces))
+
+(autoload (quote show-ws-toggle-show-hard-spaces) "show-wspace" "\
+Toggle highlighting of non-breaking space characters (`\240').
+Uses face `show-ws-hard-space'." t nil)
+
+(defalias (quote toggle-show-trailing-whitespace-show-ws) (quote show-ws-toggle-show-trailing-whitespace))
+
+(autoload (quote show-ws-toggle-show-trailing-whitespace) "show-wspace" "\
+Toggle highlighting of trailing whitespace.
+Uses face `show-ws-trailing-whitespace'." t nil)
+
 ;;;***
 
 ;;;### (autoloads (sm-add-all-headers sm-add-random-header) "silly-mail"
